subroutine CalcEnergy(Pos, M, L, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    !======== YOUR CODE HERE ========
    real(8) :: d2, id2, id6, id12, Shift, rc2 
    integer :: i, j
    real(8), dimension(Dim) :: rij, Fij, Posi 
    real(8) :: Srij    
    Shift = -4.*(rc**(-12)-rc**(-6))    
    rc2 = rc*rc    
    PEnergy = 0.
    
    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        do j = i+1, NAtom -1
            rij = Pos(j,:) - Posi
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij*rij)
                       
            if (j == i+1 .and. mod(j, M)>0) then
                Srij = sqrt(d2)
                PEnergy = PEnergy + k/2.*(Srij - r0)*(Srij - r0)
            else
                if (d2 > rc2) then
                    cycle                
                end if
                id2 = 1./d2
                id6 = id2*id2*id2
                id12 = id6*id6       
                PEnergy = PEnergy + 4.*(id12-id6) + Shift
            end if
        enddo
    enddo
end subroutine

subroutine CalcEnergyForces(Pos, M, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 1.
    !======== YOUR CODE HERE ========
    real(8) :: d2, id2, id6, id12, Shift, rc2    
    integer :: i, j
    real(8), dimension(Dim) :: rij, Fij, Posi 
    real(8) :: Srij    
    Shift = -4.*(rc**(-12)-rc**(-6))    
    rc2 = rc*rc    
    PEnergy = 0.
    Forces = 0.
    
    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        
        do j = i+1, NAtom -1
            rij = Pos(j,:)-Posi
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij*rij)
                        
            if (j == i+1 .and. mod(j, M)>0) then
                Srij = sqrt(d2)
                PEnergy = PEnergy + k/2.*(Srij - r0)*(Srij - r0)
                Fij = k*(Srij-r0)*rij/Srij
                

            else
                if (d2 > rc2) then
                    cycle                
                end if

                id2 = 1./d2
                id6 = id2*id2*id2
                id12 = id6*id6       
                PEnergy = PEnergy + 4.*(id12-id6) + Shift
                Fij = rij*((-48.*id12 + 24.*id6)*id2)
            end if
            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij       
        enddo
    enddo

end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine

subroutine CalcEnergyVirial(Pos, M, L, rc, MoveAtom, PEnergy, W, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, MoveAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy, W
    real(8), parameter :: k = 3000., r0 = 1.
    !======== YOUR CODE HERE ========
    real(8) :: d2, id2, id6, id12, Shift, rc2 
    integer :: i, j
    real(8), dimension(Dim) :: rij, Posi 
    real(8) :: Srij
    
    Shift = -4.*(rc**(-12)-rc**(-6))    
    rc2 = rc*rc    
    PEnergy = 0.
    W = 0.
    i= MoveAtom
    Posi = Pos(i,:)

    do j = 0, NAtom - 1
        if (j/=i) then
            rij = Pos(j,:) - Posi
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij*rij)
                       
            if (abs(i-j)==1 .and. mod(max(i,j), M)>0) then
                Srij = sqrt(d2)
                PEnergy = PEnergy + k/2.*(Srij - r0)*(Srij - r0)
                W = W + k*(Srij-r0)*Srij
            else
                if (d2 > rc2) then
                    cycle                
                end if
                id2 = 1./d2
                id6 = id2*id2*id2
                id12 = id6*id6       
                PEnergy = PEnergy + 4.*(id12-id6) + Shift
                W = W - 24.*(2.*id12-id6) 
            end if
        end if
    enddo
   
end subroutine

subroutine CalcEnergyVirialTotal(Pos, M, L, rc, PEnergy, W, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy, W
    real(8), parameter :: k = 3000., r0 = 1.
    !======== YOUR CODE HERE ========
    real(8) :: d2, id2, id6, id12, Shift, rc2 
    integer :: i, j
    real(8), dimension(Dim) :: rij, Posi 
    real(8) :: Srij    
    Shift = -4.*(rc**(-12)-rc**(-6))    
    rc2 = rc*rc    
    PEnergy = 0.
    W= 0.
    
    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        do j = i+1, NAtom -1
            rij = Pos(j,:) - Posi
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij*rij)
                       
            if (j == i+1 .and. mod(j, M)>0) then
                Srij = sqrt(d2)
                PEnergy = PEnergy + k/2.*(Srij - r0)*(Srij - r0)
                W = W + k*(Srij-r0)*Srij
            else
                if (d2 > rc2) then
                    cycle                
                end if
                id2 = 1./d2
                id6 = id2*id2*id2
                id12 = id6*id6       
                PEnergy = PEnergy + 4.*(id12-id6) + Shift
                W = W - 24.*(2.*id12-id6) 
            end if
        enddo
    enddo
end subroutine